/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2009 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 * Contributor: Philippe Merle
 * 
 */
package org.objectweb.fractal.bf.connectors.ws;

import org.apache.cxf.aegis.AegisContext;
import org.apache.cxf.aegis.databinding.AegisDatabinding;
import org.apache.cxf.endpoint.Server;
import org.apache.cxf.frontend.ServerFactoryBean;
import org.apache.cxf.service.invoker.BeanInvoker;
import org.objectweb.fractal.bf.connectors.common.uri.UriSkeletonContent;

/**
 * The content implementation of Web Service skeleton components.
 */
public class WsSkeletonContent extends UriSkeletonContent implements
		WsSkeletonContentAttributes {

	/**
	 * The server factory bean.
	 */
	private final ServerFactoryBean wsServerFactoryBean;

	/**
	 * A reference to the server bean. This reference is initialized during the
	 * {@link WsSkeletonContent#startFc()} method.
	 */
	private Server server;

	public WsSkeletonContent() {
		wsServerFactoryBean = new ServerFactoryBean();
		/*
		 * the databinding used by this plugin. Should be this configurable,
		 * something must be done in the attribute controller and in this code.
		 */
		AegisDatabinding aDB = new AegisDatabinding();
		final AegisContext aegisContext = new AegisContext();
		aegisContext.setWriteXsiTypes(true);
		aDB.setAegisContext(aegisContext);

		wsServerFactoryBean.getServiceFactory().setDataBinding(aDB);
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	@Override
	public void startFc() {
		String uri = getUri();
		wsServerFactoryBean.setAddress(uri);
		log.fine("ServerFactoryBean will expose web service at URI: " + uri);
		wsServerFactoryBean.setServiceClass(getServiceClass());
		BeanInvoker invoker = new BeanInvoker(getServant());
		wsServerFactoryBean.setInvoker(invoker);
		this.server = wsServerFactoryBean.create();
		log.info("WSDL available at the address: " + uri + "?wsdl");
		super.startFc();
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	@Override
	public void stopFc() {
		this.server.stop();
		log.fine("WebService skeleton stopped");
		super.stopFc();
	}
}
